VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsDraw_Object"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
' The PictureBox we draw on.
Public Enum ObjTypes
  OTNone
  OTLine
  OTCircle
  OTRectangle
End Enum
#If False Then 'Trick preserves Case of Enums when typing in IDE
Private OTNone, OTLine, OTCircle, OTRectangle
#End If
Private CFm_picCanvas         As PictureBox
' The rectangle's drawing attributes.
Private PrevX                 As Single
Private PrevY                 As Single
Private StartX                As Single
Private StartY                As Single
Private CFm_X1                As Single
Private CFm_Y1                As Single
Private CFm_X2                As Single
Private CFm_Y2                As Single
Private CFm_X0                As Single
Private CFm_Y0                As Single
Private CFm_Radius            As Single
Private CFm_DrawWidth         As Integer
Private CFm_ForeColor         As OLE_COLOR
Private CFm_FillColor         As OLE_COLOR
Private CFm_FillStyle         As FillStyleConstants
' True if the object is selected.
Private CFm_ObjType           As ObjTypes
Private CFm_Selected          As Boolean
Private Const CLOSE_DIST      As Integer = 30

Private Sub Class_Initialize()

  CFm_DrawWidth = 1
  ' Set some defaults.
  CFm_ForeColor = vbBlack
  CFm_FillColor = vbWhite
  CFm_FillStyle = vbFSSolid

End Sub

Public Function DistToSegment(ByVal px As Single, _
                              ByVal py As Single, _
                              ByVal X1 As Single, _
                              ByVal Y1 As Single, _
                              ByVal X2 As Single, _
                              ByVal Y2 As Single) As Single

  Dim dy As Single
  Dim t  As Single
  Dim dx As Single

  ' Calculate the distance from a point to a segment.
  dx = X2 - X1
  dy = Y2 - Y1
  If dx = 0 Then
    If dy = 0 Then
      ' It's a point not a line segment.
      dx = px - X1
      dy = py - Y1
      DistToSegment = Sqr(dx * dx + dy * dy)
      Exit Function
    End If
  End If
  t = (dx * (px - X1) + dy * (py - Y1)) / (dx * dx + dy * dy)
  If t < 0 Then
    dx = px - X1
    dy = py - Y1
   ElseIf t > 1 Then
    dx = px - X2
    dy = py - Y2
   Else
    X2 = X1 + t * dx
    Y2 = Y1 + t * dy
    dx = px - X2
    dy = py - Y2
  End If
  DistToSegment = Sqr(dx * dx + dy * dy)

End Function

Public Sub Draw()

  ' See if the object is selected.
  ' Draw on picCanvas.

  If CFm_Selected Then
    ' It is selected.
    ' Draw in yellow to highlight it.
    picCanvas.ForeColor = vbYellow
   Else
    ' It is not selected.
    ' Use its normal ForeColor.
    picCanvas.ForeColor = CFm_ForeColor
  End If
  ' Set other drawing attributes.
  With CFm_picCanvas
    .FillStyle = CFm_FillStyle
    .FillColor = CFm_FillColor
    .DrawWidth = CFm_DrawWidth
    .DrawMode = vbCopyPen
  End With
  ' Draw the object.
  Select Case CFm_ObjType
   Case OTLine
    picCanvas.Line (CFm_X1, CFm_Y1)-(CFm_X2, CFm_Y2), picCanvas.ForeColor
   Case OTCircle
    picCanvas.Circle (CFm_X0, CFm_Y0), CFm_Radius, picCanvas.ForeColor
   Case OTRectangle
    picCanvas.Line (CFm_X1, CFm_Y1)-(CFm_X2, CFm_Y2), picCanvas.ForeColor, B
  End Select

End Sub


Public Property Get DrawWidth() As Integer

  DrawWidth = CFm_DrawWidth

End Property

Public Property Let DrawWidth(PropVal As Integer)

  CFm_DrawWidth = PropVal

End Property

Public Sub EndDrawing(ByVal X As Single, _
                      ByVal Y As Single)

  ' Create the new object.

  CFm_picCanvas.DrawMode = vbCopyPen
  Select Case CFm_ObjType
   Case OTRectangle, OTLine    ' Drawing a rectangle.
    'With Me
    Select Case CFm_ObjType
     Case OTRectangle
      ObjType = OTRectangle
     Case OTLine
      ObjType = OTLine
    End Select
    X1 = StartX
    Y1 = StartY
    X2 = PrevX
    Y2 = PrevY
    ' End With
   Case OTCircle        ' Drawing a circle.
    ' Set Obj = New draw_Line
    ObjType = OTCircle
    X0 = (StartX + PrevX) / 2
    Y0 = (StartY + PrevY) / 2
    If Abs(StartX - PrevX) <= Abs(StartY - PrevY) Then
      Radius = Abs(StartX - PrevX) / 2
     Else
      Radius = Abs(StartY - PrevY) / 2
    End If
  End Select
  Selected = True

End Sub

Public Property Get FillColor() As OLE_COLOR

  FillColor = CFm_FillColor

End Property

Public Property Let FillColor(PropVal As OLE_COLOR)

  CFm_FillColor = PropVal

End Property

Public Property Get FillStyle() As FillStyleConstants

  FillStyle = CFm_FillStyle

End Property

Public Property Let FillStyle(PropVal As FillStyleConstants)

  CFm_FillStyle = PropVal

End Property

Public Property Get ForeColor() As OLE_COLOR

  ForeColor = CFm_ForeColor

End Property

Public Property Let ForeColor(PropVal As OLE_COLOR)

  CFm_ForeColor = PropVal

End Property

Public Sub InitDrawing(X As Single, _
                       Y As Single)

  CFm_picCanvas.DrawMode = vbInvert
  FillStyle = vbFSTransparent
  DrawWidth = 1
  StartY = Y
  StartX = X
  PrevX = X
  PrevY = Y

End Sub

Public Function IsAt(ByVal X As Single, _
                     ByVal Y As Single) As Boolean

  Select Case CFm_ObjType
   Case OTLine
    IsAt = (DistToSegment(X, Y, CFm_X1, CFm_Y1, CFm_X2, CFm_Y2) < CLOSE_DIST)
   Case OTCircle
    IsAt = (CFm_X0 - X) * (CFm_X0 - X) + (CFm_Y0 - Y) * (CFm_Y0 - Y) < CFm_Radius * CFm_Radius
   Case OTRectangle
    IsAt = (CFm_X1 <= X And CFm_X2 >= X And CFm_Y1 <= Y And CFm_Y2 >= Y)
  End Select
  ' Return True if the object is at this position.

End Function

Public Property Get ObjType() As ObjTypes

  ObjType = CFm_ObjType

End Property

Public Property Let ObjType(PropVal As ObjTypes)

  CFm_ObjType = PropVal

End Property

Public Property Get picCanvas() As PictureBox

  Set picCanvas = CFm_picCanvas

End Property

Public Property Set picCanvas(PropVal As PictureBox)

  Set CFm_picCanvas = PropVal

End Property

Public Property Get Radius() As Single

  Radius = CFm_Radius

End Property

Public Property Let Radius(PropVal As Single)

  CFm_Radius = PropVal

End Property

Public Property Get Selected() As Boolean

  Selected = CFm_Selected

End Property

Public Property Let Selected(PropVal As Boolean)

  CFm_Selected = PropVal

End Property

Public Sub WhileDrawing(X As Single, _
                        Y As Single)

  Dim r As Single

  Select Case CFm_ObjType
   Case OTRectangle     ' Drawing a rectangle.
    CFm_picCanvas.Line (StartX, StartY)-(PrevX, PrevY), , B
   Case OTCircle        ' Drawing a circle.
    If Abs(StartX - PrevX) <= Abs(StartY - PrevY) Then
      r = Abs(StartX - PrevX) / 2
     Else
      r = Abs(StartY - PrevY) / 2
    End If
    CFm_picCanvas.Circle ((StartX + PrevX) / 2, (StartY + PrevY) / 2), r
   Case OTLine          ' Drawing a line.
    CFm_picCanvas.Line (StartX, StartY)-(PrevX, PrevY)
  End Select
  ' Update the last coordinates.
  PrevX = X
  PrevY = Y
  ' Draw the new object.
  Select Case CFm_ObjType
   Case OTRectangle     ' Drawing a rectangle.
    CFm_picCanvas.Line (StartX, StartY)-(PrevX, PrevY), , B
   Case OTCircle        ' Drawing a circle.
    If Abs(StartX - PrevX) <= Abs(StartY - PrevY) Then
      r = Abs(StartX - PrevX) / 2
     Else
      r = Abs(StartY - PrevY) / 2
    End If
    CFm_picCanvas.Circle ((StartX + PrevX) / 2, (StartY + PrevY) / 2), r
   Case OTLine          ' Drawing a line.
    CFm_picCanvas.Line (StartX, StartY)-(PrevX, PrevY)
  End Select

End Sub

Public Property Get X0() As Single

  X0 = CFm_X0

End Property

Public Property Let X0(PropVal As Single)

  CFm_X0 = PropVal

End Property

Public Property Get X1() As Single

  X1 = CFm_X1

End Property

Public Property Let X1(PropVal As Single)

  CFm_X1 = PropVal

End Property

Public Property Get X2() As Single

  X2 = CFm_X2

End Property

Public Property Let X2(PropVal As Single)

  CFm_X2 = PropVal

End Property

Public Property Get Y0() As Single

  Y0 = CFm_Y0

End Property

Public Property Let Y0(PropVal As Single)

  CFm_Y0 = PropVal

End Property

Public Property Get Y1() As Single

  Y1 = CFm_Y1

End Property

Public Property Let Y1(PropVal As Single)

  CFm_Y1 = PropVal

End Property

Public Property Get Y2() As Single

  Y2 = CFm_Y2

End Property

Public Property Let Y2(PropVal As Single)

  CFm_Y2 = PropVal

End Property

':)Roja's VB Code Fixer V1.1.61 (25/11/2003 12:23:46 PM) 32 + 346 = 378 Lines Thanks Ulli for inspiration and lots of code.

